﻿// Description: Title case
// Peter Kahrel

/*__________________________________________________________________________________

The script converts text to title case.
Words in the text file 'title_case_exceptions.txt' (which must be in the script folder) are ignored.

Five modes:

1. Insertion point in italic text: process just the italic text of the current paragraph.
2. Insertion point selected, not in italic text: try to find text in quotes in the current paragraph and process that.
3. Insertion point selected, not in italic text and not in quotes: do the ins. point's parent paragraph.
4. Selection is not an insertion point but some selected text: process whatever is selected.
5. Text frame selected: process the frame's parent story.
6. Nothing selected: marked paragraph styles are title-cased.



ignore = /\b(in|the|a)\b/;
colon = /:$/;

p = app.selection[0].paragraphs[0];
//p = app.selection[0];
p.changecase (ChangecaseMode.LOWERCASE)
w = p.words.everyItem().getElements();
w1 = p.words.everyItem().contents;
w[0].changecase (ChangecaseMode.TITLECASE);
for (var i = w1.length-1; i > 0; i--) {
	if (!ignore.test (w1[i]) || colon.test (w1[i-1])) {
		w[i].changecase (ChangecaseMode.TITLECASE);
	}
}

____________________________________________________________________________________*/

(function () {
	
	var ignore;
	var colon = /[:.]$/;
	var italic = /Italic|Oblique/i;

	function get_exceptions () {
		var f = File (script_dir () + '/title_case_exceptions.txt');
		if (f.exists) {
			f.open ('r');
			var temp = f.read ();
			f.close ();
			return RegExp ('\\b(' + temp.replace (/[\n\r]/g, '|') + ')\\b');
		}
		alert ('Cannot find title_case_exceptions.txt in the script folder.', 'Error', true);
		exit();
	}


	function script_dir () {
		try {
			return File (app.activeScript).path;
		} catch (e) {
			return File (e.fileName).path;
		}
	}


	// Return the italic text that an insertion points sits in

	function get_italic_text (par) {
		var first, last;
		first = last = app.selection[0].insertionPoints[0].index - par.insertionPoints[0].index;
		var ip = par.insertionPoints.everyItem().getElements();
		while (ip[first].fontStyle == 'Italic') {
			first--
		}
		while (ip[last].fontStyle == 'Italic') {
			last++;
		}
		return par.characters.itemByRange (first, last-2);
	}


	// Return text in quotation marks

	function get_quote (par) {
		// First try to match text in double quotes
		var s = par.contents;
		if (s.indexOf('\u201C') > -1 && s.indexOf('\u201D') > -1) {
			return par.characters.itemByRange (s.indexOf('\u201C')+1, s.indexOf('\u201D')-1);
		}

		// Then try single quotes
		if (s.indexOf('\u2018') > -1 && s.indexOf('\u2019') > -1) {
			return par.characters.itemByRange (s.indexOf('\u2018')+1, s.indexOf('\u2019')-1);
		}
		return null; // No quotation marks
	}


	function process_story (story) {
		app.findGrepPreferences = null;
		app.findGrepPreferences.fontStyle = 'Italic';
		var found = story.findGrep();
		for (var i = 0; i < found.length; i++) {
			apply_title_case (found[i]);
		}
	}


	// Apply title case to some selection of found text.
	// Content returned by .itemsByRange() is an array.
	// Content of a selection or of found array items are strings

	function apply_title_case (text) {
		text.changecase (ChangecaseMode.LOWERCASE);
		var words = text.words.everyItem().getElements();
		var contents = text.words.everyItem().contents;
		words[0].changecase (ChangecaseMode.TITLECASE);
		for (var i = contents.length-1; i > 0; i--) {
			if (!ignore.test (contents[i]) || colon.test (contents[i-1])) {
				words[i].changecase (ChangecaseMode.TITLECASE);
			}
		}
	}


	function titleCaseParagraphStyles () {
		var pstyles = app.documents[0].allParagraphStyles;
		
		function tcase (list) {
			for (var i = list.length-1; i >= 0; i--) {
				apply_title_case (list[i]);
			}
		}

		app.findGrepPreferences = null;
		for (var i = pstyles.length-1; i > 0; i--) {
			if (pstyles[i].name.indexOf ('title case') > 0 || pstyles[i].extractLabel ('title case') !== '') {
				app.findGrepPreferences.appliedParagraphStyle = pstyles[i];
				tcase (app.documents[0].findGrep())
			}
		}
	}

	//---------------------------------------------------------------------------------
	
	function title_case () {

		ignore = get_exceptions(); // Get the exceptions from the text file; global
		var sel;
		
		if (app.selection.length === 0) {
			titleCaseParagraphStyles();
			exit();
		}
		
		if (app.selection[0] instanceof TextFrame) {
			process_story (app.selection[0].parentStory);
			return;
		}

		// If the selection is an insertion point and the 
		// insertion point is italic, then do whatever is in italics
		
		if (app.selection[0] instanceof InsertionPoint) {
			if (italic.test (app.selection[0].fontStyle)) {
				apply_title_case (get_italic_text (app.selection[0].paragraphs[0]));
			} else {
				// If the ins. point is embraced by quotes, return that,
				// otherwise return the whole paragraph
				sel = get_quote (app.selection[0].paragraphs[0]);
				if (sel) {
					apply_title_case (sel);
				} else {
					apply_title_case (app.selection[0].paragraphs[0]);
				}
			}
		} else { // Not an insertion point, do whatever is selected
			apply_title_case (app.selection[0]);
		}
	}

	title_case();

}());